%% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %%
% <><><><><><><><>      Automated VOI Extraction        <><><><><><><><> %
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %
%                                                 Version:    29.01.2010 %
%                                                     (c) Roland Pfister %
%                                             roland_pfister@t-online.de %
% 1. Synopsis                                                            %
%                                                                        %
% Volume of Interest files are extracted for any number of regions, con- %
% trasts, sessions, and participants. The batch needs a certain folder   %
% layout to function properly:                                           %
%                                                                        %
% \...\[Experiment Folder]                                               %
%                                                                        %
%      \Batch    <<< Folder containing 'extract_voi.m'                   %
%      \pb_01    <<< Participant folders consisting of                   %                      
%      \pb_02    <<< the string entered below for the                    %
%      \[...]    <<< variable 'participant_prefix' and                   %
%      \pb_10    <<< a two-digit participant index.                      %
%      \[...]                                                            %
%                                                                        %
% Furthermore, each participant folder has to contain a results subfol-  %
% der including the individual SPM.mat. The script will automatically    %
% create a subfolder 'VOIs' in the results folder of each participant    %
% and save the respective VOI files sorted by contrast.                  %
%                                                                        %
% All other settings are explained in the script and are marked at the   %
% right margin with the symbol '<-X'.                                    %
%                                                                        %
% 2. Content (Sections)                                                  %
%                                                                        %
%       - VOI specification                                              %
%       - VOI extraction                                                 %
%       - Workspace Cleanup                                              %
%                                                                        %
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %


%% VOI specification            <><><><><><><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %
clear all;
% Design specification and folder layout.
participants = [1];   % Participant and contrast indices; the input       <-X (1/11)
%Selection: [1:3,5,6,9,11:20,22,23]
%BA7 geht nicht bei:9, 12, 15, 16, 17, 22 [1:6,11,13,14,18:20,23
%BA40 geht nicht bei: 9, 12, 15, 17, 22 [1:6,11,13:14,16,18:20,22,23]
contrasts = [34];        % has to be a column vector, e.g. [1:3,5,10].       <-X (2/11)
n_sessions = 3;         % Number of sessions in the design (integer).       <-X (3/11)

base_folder = strrep(pwd,'Batch','');
addpath(pwd);
participant_prefix = 'pb_';             % Participant folders prefix.       <-X (4/11)
results_folder = 'results_resp\';       % Individual results folders.       <-X (5/11)


% VOI names and respective coordinates for each contrast, e.g.              <-X (6/11) 
%
%    target_regions{1} = { [-36; 17; -2], 'Insula_L'; ...
%                          [ 36; 17; -2], 'Insula_R'; };
%    target_regions{2} = { [ 36; 17; -2], 'Insula_R'; };
%
% ... to extract VOIs for the left and right insula in con-
% trast #1 and a VOI for the right insula in contrast #2.
            
target_regions{34} = { [ 36; 17; -2], 'Insula_R_v2'; };


voi_type = 'sphere';   % Type of VOI - currently, only                      <-X (7/11) 
                       % spheres are supported.
voi_size = 10;         % Radius of the sphere.                              <-X (8/11) 
con_p    = 0.05;       % p-threshold for contrast evaluation.               <-X (9/11) 
con_k    = 0;          % Minimum cluster size.                              <-X (10/11)
nearest  = 0;          % 0 = use the xyz values above.                      <-X (11/11)
                       % 1 = use nearest local maximum.
                       % Note that the batch will crash if 
                       % 'nearest' is set to 0 and no supra-
                       % threshold voxels are found in the VOI.


%% VOI extraction           <><><><><><><><><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %

fprintf('\n\n');
fprintf(['<><><><><><><><><><><><><><><><><><><><><><><><><><><><><>' ...
    '<><><><><><>\n<><><><><><><><>         Starting VOI Extraction ' ...
    '     <><><><><><><><>\n<><><><><><><><><><><><><><><><><><><><>' ...
    '<><><><><><><><><><><><><><><>\n']);

for participant_index = 1:size(participants,2)
    
participant = participants(participant_index);
if participant < 10
    prefix_correction = '0';
else
    prefix_correction = '';
end;

fprintf('\n\n');
fprintf('<> Participant: %s\n\n',[prefix_correction ...
    num2str(participant) ...
    '   <><><><><><><><><><><><><><><><><><><><><><><><>']);

participant_directory = [base_folder participant_prefix ...
    prefix_correction num2str(participant) '\'];

cd(participant_directory)
cd(results_folder)

try
    cd VOIs
catch
    mkdir('VOIs');
    cd VOIs
end;

voi_directory = pwd;

for contrast_index = 1:size(contrasts,2)

cd(voi_directory)

contrast = contrasts(contrast_index);
if contrast < 10
    contrast_correction  = '00';
elseif contrast < 100
    contrast_correction  = '0';  
else
    contrast_correction  = '';  
end;

fprintf('\t< Contrast: %s >\n',[contrast_correction num2str(contrast)]);

try
    cd(['con_' contrast_correction num2str(contrast)])
catch
    mkdir(['con_' contrast_correction num2str(contrast)]);
    cd(['con_' contrast_correction num2str(contrast)])
end;

contrast_directory = pwd;

% Get SPM.mat
% Evaluated fields in xSPM (input)
% xSPM      - structure containing SPM, distribution & filtering details
% .swd      - SPM working directory - directory containing current SPM.mat
% .title    - title for comparison (string)
% .Ic       - indices of contrasts (in SPM.xCon)
% .Im       - indices of masking contrasts (in xCon)
% .pm       - p-value for masking (uncorrected)
% .Ex       - flag for exclusive or inclusive masking
% .u        - height threshold
% .k        - extent threshold {voxels}
% .thresDesc - description of height threshold (string)
%
xSPM.swd    = [participant_directory results_folder];
xSPM.title  = ['VOI_con_' num2str(contrast)];
xSPM.Ic     = contrast;
xSPM.Im     = [];
xSPM.pm     = [];
xSPM.Ex     = [];
xSPM.u      = con_p;
xSPM.k      = con_k;
xSPM.thresDesc = 'none';


%cd(spm('Dir')); % Only needed if WFU_Pickatlas is installed.
%[SPM,xSPM] = spm_getSPM(xSPM);
%[SPM,xSPM] = spm_getSPM();
addpath(pwd);
[SPM,xSPM] = extract_voi_spm5_spm_getSPM(xSPM);



%-Get volumetric data from SPM.mat
%-----------------------------------------------------------------------
try
    xX   = SPM.xX;                  %-Design definition structure
    XYZ  = SPM.xVol.XYZ;			%-XYZ coordinates
    S    = SPM.xVol.S;              %-search Volume {voxels}
    R    = SPM.xVol.R;              %-search Volume {resels}
    M    = SPM.xVol.M(1:4,1:4);		%-voxels to mm matrix
    VOX  = sqrt(diag(M'*M))';		%-voxel dimensions
catch
    error ('This model has not been estimated.');
end
                
                

for region_index = 1:size(target_regions{contrast},1)

voi_name = target_regions{contrast}{region_index,2}; 
voi_xyz = target_regions{contrast}{region_index,1}; 


for session = 1:n_sessions;


    % spm_regions.m / spm_ROI.m
        
    % xY     - VOI structure
    try
        xY;
    catch %#ok<*CTCH>
        xY = {};
    end

    if nearest == 1
    %-Find nearest voxel [Euclidean distance]
        if ~length(xSPM.XYZmm) %#ok<*ISMT>
            spm('alert!','No suprathreshold voxels!',mfilename,0);
            Y = []; xY = [];
            return
        end
        [xyz,i] = spm_XYZreg('NearestXYZ',voi_xyz,xSPM.XYZmm);
        xY.xyz  = xyz;
    else
        xyz = voi_xyz;
        xY.xyz = voi_xyz;
    end;
    xY.name = voi_name;
    xY.Ic = 0;

        
    %-if fMRI data get sessions and filtering options
    if isfield(SPM,'Sess')
        s = length(SPM.Sess);
        if s > 1
            s = session;
        end
        xY.Sess   = s;
    end
    xY.rej = {};
   
        
    %-Specify VOI
    if ~isfield(xY,'def')
        xY.def = voi_type;
    end
    switch xY.def

        case 'sphere'
             if ~isfield(xY,'spec')
                xY.spec = voi_size;
            end
        otherwise
        error('Unknown or unsupported VOI type.');
    end

    xY.M = xSPM.M;
    [xY, xY.XYZmm, Q] = extract_voi_spm5_spm_ROI(xY, xSPM.XYZmm);
    try  %#ok<*TRYNC>
        xY = rmfield(xY,'M');
    end   

    %-Get raw data, whiten and filter
    y        = spm_get_data(SPM.xY.VY,xSPM.XYZ(:,Q));
    y        = spm_filter(SPM.xX.K,SPM.xX.W*y);

    % remove null space of contrast
    if xY.Ic
        cd([participant_directory results_folder]);
            %-Parameter estimates: beta = xX.pKX*xX.K*y
            beta  = spm_get_data(SPM.Vbeta,xSPM.XYZ(:,Q));

            %-subtract Y0 = XO*beta,  Y = Yc + Y0 + e
            y     = y - spm_FcUtil('Y0',SPM.xCon(xY.Ic),SPM.xX.xKXs,beta);
     end

    % confounds
    xY.X0     = SPM.xX.xKXs.X(:,[SPM.xX.iB SPM.xX.iG]);

	% extract session-specific rows from data and confounds
    try
        i     = SPM.Sess(xY.Sess).row;
        y     = y(i,:);
        xY.X0 = xY.X0(i,:);
    end

    % and add session-specific filter confounds
    try
        xY.X0 = [xY.X0 SPM.xX.K(xY.Sess).X0];
    end

    try
        xY.X0 = [xY.X0 SPM.xX.K(xY.Sess).KH]; % Compatibility check
    end
        
    % Remove null space of X0
    xY.X0     = xY.X0(:,any(xY.X0));


    % compute regional response in terms of first eigenvariate
    [m n]   = size(y);
    if m > n
        [v s v] = svd(spm_atranspa(y)); %%%svd(y'*y);
        s       = diag(s);
        v       = v(:,1);
        u       = y*v/sqrt(s(1));
    else
        [u s u] = svd(spm_atranspa(y')); %%%svd(y*y');
        s       = diag(s);
        u       = u(:,1);
        v       = y'*u/sqrt(s(1));
    end
    d       = sign(sum(v));
    u       = u*d;
    v       = v*d;
    Y       = u*sqrt(s(1)/n);


    % set in structure
    xY.y    = y;
    xY.u    = Y;
    xY.v    = v;
    xY.s    = s;
    
    % Additional information.
    xY.con_p = con_p;
    xY.con_k = con_k;
    xY.nearest = nearest;

    % save
    str     = ['VOI_' xY.name];
    if isfield(xY,'Sess')
        if length(xY.Sess) == 1
            str = sprintf('VOI_%s_%i',xY.name,xY.Sess);
        end
    end
    fprintf('\t\tSaving file: %s\n',str);
    save(fullfile(contrast_directory,str),'Y','xY')

    % Clean up.
    clear Q;
    clear d;
    clear i;
    clear m;
    clear n;
    clear s;
    clear str;
    clear u;
    clear v;
    clear xyz;
    clear y;
    
    clear Y;
    clear xY;
          
end; % Session loop.

% Clean up.
clear session;

end; % Regions loop.

% Clean up.
clear region_index;
clear voi_name; 
clear voi_xyz; 

end; % Contrasts loop.

% Clean up.
clear contrast;
clear contrast_index;
clear contrast_correction;
clear contrast_directory;
clear SPM;
clear xSPM;
clear xX;
clear XYZ;
clear S;
clear R;
clear M;
clear VOX;
clear voi_directory;

end; % Participant loop.

%% Workspace cleanup            <><><><><><><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %
clear participant;
clear prefix_correction;
clear participant_directory;
clear participant_index;

clear base_folder;
clear con_k;
clear con_p;
clear contrasts;
clear n_sessions;
clear nearest;
clear participant_prefix;
clear participants;
clear results_folder;
clear target_regions;
clear voi_size;
clear voi_type;
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %