%% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %%
% <><><><><><><><><><>   Model & Contrasts Batch    <><><><><><><><><><> %
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %
%                                                 Version:    14.09.2009 %
%                                                     (c) Roland Pfister %
%                                             roland_pfister@t-online.de %
% 1. Synopsis                                                            %
%                                                                        %
% This SPM8 batch contains automated routines for fMRI model specifica-  %
% tion, including condition and onset settings. Furthermore, contrasts   %
% a function for implementing a set of contrasts into the results file   %
% that is specific to the study it was originally designed for (> Haupt- %
% studie; Diplomarbeit).                                                 %
%                                                                        %
% To append this script to Preprocessing.m, simply change the field in-  %
% dices matlabbatch{i} to bring them in a contiunous order. Most of the  %
% settings below can be deleted in this case (see the Settings section   %
% Preprocessing.m for details).                                          %
%                                                                        %
% 2. Content (Sections)                                                  %
%                                                                        %
%       - Settings: Design- and batch specifications.                    %
%       - Start Bacth Processing                                         %
%       - Job 01: fMRI Model Specification                               %
%       - Job 02: Model Estimation                                       %
%       - Job 03: Run Contrast_Definition.m *                            %
%       - Job 04: Print Graphics Output                                  %
%       - End Batch and Workspace Cleanup                                %
%                                                                        %
% *) This job requires the Run M File addin for the batch editor of SPM8 %
%    (but can be run manually if Run M File is not available).           %
%                                                                        %
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %





%% Settings     <><><><><><><><><><><><><><><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %


% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %
% Basic Design Specifications.                                           %
%                                                                        %
TR = 2;                     % TR in seconds.                                <-X (1/10)

n_sessions = 3;             % Number of sessions.                           <-X (2/10)
starting_image = [1, 1 ,1]; % Starting and ending image indices             <-X (3/10)
final_image = [381,381,381];% for each session.                             <-X (4/10)

timing_units = 'scans';     % 'scans' or 'seconds'.                         <-X (5/10)
condition_file = ...        % Mat-file containing the following cell        <-X (6/10)
     'Onsets\Tr*.mat';      % arrays: 'onset_labels' as an 1xn array
                            % with condition names and one cell array
                            % per condition named 'onsets_session_i'
                            % with onset times.
results_dir = 'results';    % Output directory for SPM.mat.                 <-X (7/10)
jobs_dir    = 'jobs';       % Source for SPM jobs/batches.                  <-X (8/10)
serial_corr = 'AR(1)';      % 'AR(1)' or 'none'.
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %



% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %
% Image Name Specification.                                              %
%                                                                        %
% The image (EPIs) filenames and paths are concatenated from these       %
% sources:                                                               %
%                                                                        %
%   (1) base_folder         - Path of participant folder.                %
%   (2) session_names       - Folder specification for session i.        %
%   (3) participant_prefix  - Participant-specific prefix of each .img.  %
%   (4) image number        - A four-digit image number (leading 0s are  %
%                             added automatically)).                     %
%   (5) image suffix        - '.img,1'.                                  %
%                                                                        %
% Example:                                                               %
%                                                                        %
%   'C:\SampleStudy\participant_01\session_1\sn_001_0005.img'            %
%    |------------(1)------------| |--(2)--| |-(3)-||-(4,5)-|            %
%                                                                        %
cd ..;                      % The experiment folder is set to the 
exp_folder = [pwd '\'];     % directory one level above this file.
participant_folders = 'pb*';% Identifier for participant folders in
                            % the main directory of the experiment.
                            % Used to construct base_folder for each
                            % participant during batch processing.
session_names = 'session_'; % Name of session folders (excl. numbers).      <-X ( 9/10)
participant_prefix = ...    % Enter 'folder_name' to use the name of        <-X (10/10)
   'folder_name';           % the participant folder instead.
part_image_sep = '_';       % This symbol will be placed between par-
                            % ticipant number and image index in the
                            % image filenames.
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %



% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %
% Image Prefixes                                                         %
%                                                                        %
prefix_coreg     = 'r';     %T1
prefix_realign   = 'u';     %EPIs
prefix_st        = 'a';     %EPIs
prefix_normalise = 'w';     %EPIs
prefix_smooth    = 's';     %EPIs
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %

   

% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %
% Set the Stage                                                          %
%                                                                        %
clear matlabbatch;  % Clear batch variable in workspace.
j_c = 0;            % Initialise job counter.
condition_mode = 1; % 1 = Unset error condition onsets are set to 0, 1, 
                    % or 2 for condition 14, 15, and 16, respectively.
                    % 2 = Unset error condition onsets are deleted from
                    % multiple conditions .mat-file.
 
% Prepare batch for processing multiple participants. To avoid the com- 
% command line prompt, use the following lines instead (you can comment 
% the rest of the section).
% start_participant = 1;
% final_participant = 18;

start_participant = input(['\n\n\n' ...
    '<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>' ...
        '<><><><><>\n'...
    '<><><><><>      Preprocessing Batch - (c) Roland Pfister      ' ...
        '<><><><><>\n'...
    '<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>' ...
        '<><><><><>\n'...
    '<>                                                            ' ...                                                        
        '        <>\n'...
    '<> Specify the participants to be processed. Give first and '   ...
        'last fol- <>\n'...
    '<> der number starting with participant prefix. See the file'   ...
        ' for more <>\n' ...
    '<> information.                                               ' ...                                                        
        '        <>\n'...
    '<>                                                            ' ...                                                        
        '        <>\n\n'...
    'First participant        :        ']);
if ~isnumeric(start_participant)
    disp(' ');
    disp('Please enter a single integer!');
    disp(' ');    
end;
final_participant = input('Last participant         :        ');
if ~isnumeric(final_participant) 
    disp(' ');
    disp('Please enter a single integer!');
    disp(' ');   
end;
start_participant = start_participant + 1;
final_participant = final_participant + 1;
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %



%% Start Batch Processing     <><><><><><><><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %
% Get participant directories.
participants = dir([exp_folder participant_folders]);
participant_prefix_setting = participant_prefix;

for participant_index = start_participant:final_participant

base_folder = [exp_folder participants(participant_index).name  '\'];



%Get participant folder name and correct prefix:
if strcmp(participant_prefix_setting,'folder_name') == 1
    participant_prefix = [participants(participant_index). ...
        name part_image_sep];
else
    participant_prefix = [participant_prefix_setting part_image_sep];
end;
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %





%% Job 01: fMRI Model Specification     <><><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %
j_c = j_c + 1;
matlabbatch{j_c}.spm.stats.fmri_spec.dir = ...
    {[base_folder results_dir '\']};
matlabbatch{j_c}.spm.stats.fmri_spec.timing.units = timing_units;
matlabbatch{j_c}.spm.stats.fmri_spec.timing.RT = TR;
matlabbatch{j_c}.spm.stats.fmri_spec.timing.fmri_t = 16;
matlabbatch{j_c}.spm.stats.fmri_spec.timing.fmri_t0 = 1;

% Prepare multiple conditions files.
cd([base_folder jobs_dir]);
temp = dir(condition_file); % The condition .mat is read and added to a
cond_path = [base_folder jobs_dir '\Onsets\' temp.name]; % ... complete path.     
clear temp;
load(cond_path);
for i = 1:n_sessions
    names = onset_labels; %#ok<*NASGU>
    eval(['onsets = onsets_session_' num2str(i) ';']);
    if condition_mode == 1
        if isempty(onsets{1,17})
            onsets{1,17} = 0;
        end;
        if isempty(onsets{1,18})
            onsets{1,18} = 1;
        end;
        if isempty(onsets{1,19})
            onsets{1,19} = 2;
        end;
        eval(['durations = cell(1,size(onsets_session_' num2str(i) ...
            ',2));']);
    elseif condition_mode == 2
        missing_conditions = 0;
        if isempty(onsets{1,17})
            onsets(1,17:18) = onsets(1,18:19);
            onsets = onsets(1,1:18);
            names(1,17:18) = names(1,18:19);
            names = names(1,1:18);
            missing_conditions = missing_conditions + 1;
        end;
        if isempty(onsets{1,18-missing_conditions})
            onsets(1,18-missing_conditions) = ...
                onsets(1,19-missing_conditions);
            onsets = onsets(1,1:18-missing_conditions);
            names(1,18-missing_conditions) = ...
                names(1,19-missing_conditions);
            names = names(1,1:18-missing_conditions);
            missing_conditions = missing_conditions + 1;
        end;
        if isempty(onsets{1,19-missing_conditions})
            onsets = onsets(1,1:18-missing_conditions);
            names = names(1,1:18-missing_conditions);
        end;
        durations = cell(1,size(onsets,2));
    end;    
    durations(:) = {0};
    save(['Onsets/onsets_session_' num2str(i) '.mat'],...
        'names','onsets','durations');
end;
cd(base_folder);

% The data cell array is filled with images.
for i = 1:n_sessions
    matlabbatch{j_c}.spm.stats.fmri_spec.sess(i).scans{ ... 
        final_image(i)-starting_image(i)+1,1} = 'preallocation';
end;
for i = 1:n_sessions
    n = 1;
    matlabbatch{j_c}.spm.stats.fmri_spec.sess(i).cond = struct(...
        'name', {}, 'onset', {}, 'duration', {}, 'tmod', {}, 'pmod', {});
    % Multiple conditions file.
    matlabbatch{j_c}.spm.stats.fmri_spec.sess(i).multi = {[base_folder ...
        jobs_dir '\Onsets\' 'onsets_session_' num2str(i) '.mat']};
    matlabbatch{j_c}.spm.stats.fmri_spec.sess(i).regress = ...
        struct('name', {}, 'val', {});
    matlabbatch{j_c}.spm.stats.fmri_spec.sess(i).multi_reg = {''};
    matlabbatch{j_c}.spm.stats.fmri_spec.sess(i).hpf = 128;
for j = starting_image(i):final_image(i)
    if j < 10
        temp = '000';
    elseif j < 100
        temp = '00';
    else
        temp = '0';
    end;
    matlabbatch{j_c}.spm.stats.fmri_spec.sess(i).scans{n,1} = ...
    [base_folder session_names num2str(i) '\' prefix_smooth  ...
    prefix_normalise prefix_st prefix_realign participant_prefix ...
    temp num2str(j) '.img,1']; %#ok<*SAGROW>
    n = n + 1;
end;
end;

matlabbatch{j_c}.spm.stats.fmri_spec.fact = ...
    struct('name', {}, 'levels', {});
matlabbatch{j_c}.spm.stats.fmri_spec.bases.hrf.derivs = [0 0];
matlabbatch{j_c}.spm.stats.fmri_spec.volt = 1;
matlabbatch{j_c}.spm.stats.fmri_spec.global = 'None';
matlabbatch{j_c}.spm.stats.fmri_spec.mask = {''};
matlabbatch{j_c}.spm.stats.fmri_spec.cvi = serial_corr;
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %





%% Job 02: Model Estimation       <><><><><><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %
j_c = j_c + 1;
matlabbatch{j_c}.spm.stats.fmri_est.spmmat = ...
    {[base_folder results_dir '\SPM.mat']};
matlabbatch{j_c}.spm.stats.fmri_est.method.Classical = 1;
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %





%% Job 03: Run Contrast_Definition.m      <><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %
j_c = j_c + 1;
matlabbatch{j_c}.cfg_zoppo{1}.run_m.mfile = ...
    {[base_folder jobs_dir '\Contrast_Definition.m']};
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %





%% Job 04: Run Contrast_Definition.m      <><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %
j_c = j_c + 1;
matlabbatch{j_c}.spm.util.print.fname = [base_folder results_dir ...
    '\output_' results_dir '.ps'];
matlabbatch{j_c}.spm.util.print.opts.opt = {
                                          '-dpsc2'
                                          '-append'
                                          }';
matlabbatch{j_c}.spm.util.print.opts.append = true;
matlabbatch{j_c}.spm.util.print.opts.ext = '.ps';
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %





%% End Batch and Workspace Cleanup.         <><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %

% End Batch Processing.
end;

% Clear preprocessing duplicates.
clear TR;
clear base_folder;
clear final_image;
clear i;
clear j;
clear n;
clear n_sessions;
clear part_image_sep;
clear participant_prefix;
clear prefix_coreg;
clear prefix_normalise;
clear prefix_realign;
clear prefix_smooth;
clear prefix_st;
clear prompt_mode;
clear reference_slice;
clear session_names;
clear starting_image;
clear subject_number;
clear temp;

% Clear unique model variables.
clear cond_path;
clear condition_file;
clear durations;
clear jobs_dir;
clear names;
clear onset_labels;
clear onsets;
clear onsets_session_1;
clear onsets_session_2;
clear onsets_session_3;
clear results_dir;
clear serial_corr;
clear timing_units;
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %
%cd jobs;