%% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %%
% <><><><><><><>      Second Level Analysis (Batch)       <><><><><><><> %
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %
%                                                 Version:    15.10.2009 %
%                                                     (c) Roland Pfister %
%                                             roland_pfister@t-online.de %
% 1. Synopsis                                                            %
%                                                                        %
% Contrast images (con_*.img) of a set of participants are combined to a %
% new model for group analysis. This analysis is carried out for (a) all %
% contrasts and (b) for all analysis types (essentially: response-locked %
% vs. target-locked), and also (c) either with or without parametric mo- %
% dulation (compatibility effect).                                       %
%                                                                        %
% The models (SPM.mat) are saved into separate folders and are estimated %
% indepedently. A t-contrast (vector: [ 1 ]) is added to SPM.mat and is  %
% evaluated instantly.                                                   %
%                                                                        %
% 2. Content (Sections)                                                  %
%                                                                        %
%       - Gather model information                                       %
%       - Job 1: Model specification                                     %
%       - Job 2: Model estimation                                        %
%       - Add Contrast Structure to SPM.mat                              %
%       - Workspace Cleanup                                              %
%                                                                        %
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %

% Get experiment folder.
batch_folder = pwd;
cd ..;
exp_folder = pwd;
exp_folder = [exp_folder '\'];
cd(batch_folder);

if exist([batch_folder '\job_03_Second_Level_Analysis_temp.mat'], ...
        'file')
    run_count = 1;
else
    run_count = 0;
end;
    
if run_count == 1
    
    load([batch_folder '\job_03_Second_Level_Analysis_temp.mat']);
    
    % Contrast labels.
    clear contrasts;

 ncontrasts = 119;

% Contrast titles.
contrasts(1,1).name = 'Free_ges vs. Forced_ges';
contrasts(1,2).name = 'Free_left vs. Forced_left';
contrasts(1,3).name = 'Free_right vs. Forced_right';



else

   
    batch_mode = 1;

    participants = ...
         {'01','02','03'};

    start_contrast = 1;
    final_contrast = 1; %ncontrasts;
    start_analysis = 2;
    final_analysis = 2;
    start_parametric = 1;
    final_parametric = 1;

   
end;

% Folder names.
analysis_folders = {'group\','group_parametric\'};
results_folders = {'results\','results_resp\'};
% Initialise job counter.
j_c = 0;

for contrast = start_contrast:final_contrast
for analysis = start_analysis:final_analysis
for parametric = start_parametric:final_parametric


results_folder = results_folders{analysis};

if contrast < 10
    con_suffix = '000';
elseif contrast < 100
    con_suffix = '00';
elseif contrast < 1000
    con_suffix = '0';
else
    con_suffix = '';
end;
con_folder = ['con_' con_suffix num2str(contrast) '\'];

analysis_folder = analysis_folders{parametric};
if parametric == 2
    parametric_vector = [1;2;3];
end;

cd(exp_folder);
if exist(analysis_folder,'dir') < 7
    mkdir(analysis_folder);
end;
cd([exp_folder analysis_folder]);
if exist(results_folder,'dir') < 7
    mkdir(results_folder);
end;
cd(results_folder);
if run_count == 0
if exist(con_folder,'dir') == 7
    rmdir(con_folder);
end;
end;
if exist(con_folder,'dir') < 7
    mkdir(con_folder);
end;
cd(batch_folder);

if run_count == 0 % ... then load model and estimation jobs.

    

%% Model Specification          <><><><><><><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %
j_c = j_c + 1;
matlabbatch{j_c}.spm.stats.factorial_design.dir = {[exp_folder ...
                                                    analysis_folder ...
                                                    results_folder ...
                                                    con_folder]}; ...
                                                    %#ok<*SAGROW>
for pb = 1:size(participants,2)
matlabbatch{j_c}.spm.stats.factorial_design.des.t1.scans{pb} = ...
    [exp_folder 'pb_' participants{pb} '\' results_folder 'con_' ...
    con_suffix num2str(contrast) '.img,1'];
end;
matlabbatch{j_c}.spm.stats.factorial_design.cov = ...
    struct('c', {}, 'cname', {}, 'iCFI', {}, 'iCC', {});
matlabbatch{j_c}.spm.stats.factorial_design.masking.tm.tm_none = 1;
matlabbatch{j_c}.spm.stats.factorial_design.masking.im = 1;
matlabbatch{j_c}.spm.stats.factorial_design.masking.em = {''};
switch parametric
case 1
matlabbatch{j_c}.spm.stats.factorial_design.globalc.g_omit = 1;
case 2
matlabbatch{1}.spm.stats.factorial_design.globalc.g_user.global_uval = ...
    parametric_vector;
end;
matlabbatch{j_c}.spm.stats.factorial_design.globalm.gmsca.gmsca_no = 1;
matlabbatch{j_c}.spm.stats.factorial_design.globalm.glonorm = 1;
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %





%% Model Estimation         <><><><><><><><><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %
j_c = j_c + 1;
matlabbatch{j_c}.spm.stats.fmri_est.spmmat = ...
    {[exp_folder analysis_folder results_folder con_folder 'SPM.mat']};
matlabbatch{j_c}.spm.stats.fmri_est.method.Classical = 1;
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %


elseif run_count == 1 % ... add contrasts to SPM.mat.


%% Add Contrast to SPM.mat      <><><><><><><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %
clear SPM;
load([exp_folder analysis_folder results_folder con_folder 'SPM.mat']);
if ~exist([exp_folder analysis_folder results_folder con_folder ...
        'SPM.backup.mat'],'file')
save([exp_folder analysis_folder results_folder con_folder ...
        'SPM.backup.mat'],'SPM');
end;

% Define dummy-contrast for fc_util (see mailing list 
% comment by Iris Trinkler, 03.02.2005 (Item# 018719)).
try
    iX0  = [SPM.xX.iB SPM.xX.iG];
catch %#ok<*CTCH>
    iX0  = [];
end
SPM.xCon = spm_FcUtil('Set','effects of interest','F','iX0',...
    iX0,SPM.xX.xKXs);

% Important: It seems that contrasts(1,i).c has to be a row vector 
% and not a column vector like described in the mailing list.
SPM.xCon(1,1).name = ['con_' con_suffix contrast '_' ...
    contrasts(contrast).name];
SPM.xCon(1,1).STAT = 'T';
SPM.xCon(1,1).c    = 1;
SPM.xCon(1,1).iX0  = 'c';
SPM.xCon(1,1) = spm_FcUtil('Set',contrasts(1,contrast).name,...
   'T','c',1,SPM.xX.xKXs); 
spm_contrasts(SPM,1);
save([exp_folder analysis_folder results_folder con_folder ...
        'SPM.mat'],'SPM');
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %

end;    % run_count

end;    % Parametric loop.
end;    % Analysis loop.
end;    % Contrast loop.





%% Save and Clean up            <><><><><><><><><><><><><><><><><><><><> %
% ---------------------------------------------------------------------- %
%                                                                        %
if run_count == 1
    delete([batch_folder '\job_03_Second_Level_Analysis_temp.mat']);
    cd(batch_folder);
elseif run_count == 0
    
    % Last job: Re-run.
    j_c = j_c + 1;
    matlabbatch{j_c}.cfg_zoppo{1}.run_m.mfile = ...
        {[batch_folder '\job_03_Second_Level_Analysis.m']};
    
    % Save temporary variables.
    save([batch_folder '\job_03_Second_Level_Analysis_temp.mat'], ...
        'start_contrast','final_contrast','start_analysis', ...
        'final_analysis','start_parametric','final_parametric',...
        'participants');
end;

clear analysis;
clear analysis_folder;
clear analysis_folders;
clear batch_folder;
clear batch_mode;
clear con_folder;
clear con_suffix;
clear contrast;
clear exp_folder;
clear final_analysis;
clear final_contrast;
clear final_parametric;
clear j_c;
clear parametric;
clear participants;
clear pb;
clear results_folder;
clear results_folders;
clear run_count;
clear start_analysis;
clear start_contrast;
clear start_parametric;
% <><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><> %